/*
  Extension       HelpRepair
  Author          Carlo Hogeveen
  Website         eCarlo.nl/tse
  Compatibility   TSE v4 upwards, all TSE variants
  Version         1   15 Nov 2022


  This extension repairs or enhances the bullet character in TSE's Help.

  It makes TSE's Help show a bullet character if it did not,
  and for all code pages it makes Console TSE show a BIG bullet character.


  In TSE's Help without this extension:

  - Newer GUI TSE versions display a bullet character as a space character,
    making it invisible. Instead of at bullet-pointed text we were looking at
    indented paragraphs. This was the reason for this extension.

  - Older GUI TSE versions displayed a bullet character as a badly visible
    double-quote-like character.

  - Newer Linux TSE versions display the bullet character as an accented "u".

  - How Console TSE versions display a bullet character depends on the code
    page of its command prompt. Type "chcp" if you are curious. Typically only
    the USA uses code page 437, which does show a tiny bullet. The rest of us
    did not see a bullet character in Console TSE's Help.


  INSTALLATION

  Copy this file to TSE's "mac" folder and compile it there, for example by
  opening it in TSE and applying the Macro, Compile menu.

  Add "HelpRepair" to TSE's Macro, AutoLoad List menu, and restart TSE.


*/


#ifdef LINUX
  #define WIN32 FALSE
#endif


// Global constants
#define NEW_BULLET_CHAR_FOR_CONSOLE         7
#define NEW_BULLET_CHAR_FOR_GUI_AND_LINUX 149
string  OLD_BULLET_CHAR         [1] = Chr(249)


// Global variables
string new_bullet_char [1] = ''


proc help_repair()
  string  attrs [MAXSTRINGLEN] = ''
  integer bullet_pos           = 0
  string  chars [MAXSTRINGLEN] = ''
  integer old_Cursor           = Set(Cursor, OFF)
  integer window_cols          = Query(WindowCols)
  integer window_row           = 0
  integer window_x1            = Query(WindowX1)
  integer window_x1_minus_1    = window_x1 - 1

  for window_row = Query(WindowY1) to Query(WindowY1) + Query(WindowRows) - 1
    GetStrAttrXY(window_x1, window_row, chars, attrs, window_cols)
    bullet_pos = Pos(OLD_BULLET_CHAR, chars)
    if bullet_pos
      PutStrAttrXY(window_x1_minus_1 + bullet_pos, window_row,
                   new_bullet_char, attrs[bullet_pos])
    endif
  endfor

  Set(Cursor, old_Cursor)
end help_repair


proc nonedit_idle()
  UnHook(nonedit_idle)
  help_repair()
end nonedit_idle


proc after_nonedit_command()
  if DisplayMode() == _DISPLAY_HELP_
    Hook(_NONEDIT_IDLE_, nonedit_idle)
  endif
end after_nonedit_command


proc WhenPurged()
  // Proc needed when adding zz_log for debugging.
end WhenPurged


proc WhenLoaded()
  if     isGUI()
  or not WIN32
    new_bullet_char = Chr(NEW_BULLET_CHAR_FOR_GUI_AND_LINUX)
  else
    new_bullet_char = Chr(NEW_BULLET_CHAR_FOR_CONSOLE)
  endif
  Hook(_AFTER_NONEDIT_COMMAND_, after_nonedit_command)
end WhenLoaded

